function frequentPatterns = undirectedSequenceMining(data, sMin)

frequentPatterns = patternSearch(data, sMin, @initializeWords, @expandSequences, @findSupport, @sortAlphabet, @prune);

% For undirected sequences, initialization consists of creating a list
% containing the empty sequence and all sequences of length 1
function initial = initializeWords(sortedAlphabet)

emptyWord.sequence = {};
emptyWord.symmetryFlag = 1;
initial = {emptyWord};
for i = 1:size(sortedAlphabet,2)
    iw.sequence = {sortedAlphabet{i}};
    iw.symmetryFlag = 1;
    initial{end+1} = iw;
end

% This function should be implemented by students.
% - Cell arrays oldPatterns and nextPatterns shoukd have the same structure:
%   they should contain structures with two items: "sequence" and "symmetryFlag".
% - This is an example how to construct such a structure:
%       str1.sequence = {'a', 'b'}; //i.e. str1 is a pattern ab
%       str1.symmetryFlag = 0; //i.e. str1 is not a palindrome
% - Cell array alphabet contains symbols from the alphabet used in the
%   sequence and it is lexicographically sorted.
function nextPatterns = expandSequences(oldPatterns, alphabet)



% This function computes support
% - Structure "pattern" has only two items: "sequence" and "symmetryFlag".
% - The function should return support of "pattern" as described in the
%   assignment for this lecture.
function support = findSupport(pattern, data)
support = 0;
seq = data.sequence;
if isempty(pattern.sequence)
    support = support+size(data.sequence,2)-size(pattern.sequence,2)+1;
else
    patternsize = size(pattern.sequence,2);
    for i = 1:(size(seq,2)-size(pattern.sequence,2)+1)
        for j = 1:patternsize
            if seq{i+j-1} ~= pattern.sequence{j}
                break;
            end
            if j == patternsize
                support = support+1;
            end
        end
    end
end
seq = seq(end:-1:1);
if isempty(pattern.sequence)
    support = support+size(data.sequence,2)-size(pattern.sequence,2)+1;
else
    patternsize = size(pattern.sequence,2);
    for i = 1:(size(seq,2)-size(pattern.sequence,2)+1)
        for j = 1:patternsize
            if seq{i+j-1} ~= pattern.sequence{j}
                break;
            end
            if j == patternsize
                support = support+1;
            end
        end
    end
end
support = support /(2*(size(data.sequence,2)-size(pattern.sequence,2)+1));

function sortedAlphabet = sortAlphabet(data)
sortedAlphabet = sort(unique(data.sequence));

function filtered = prune(shorter, longer)
f1 = prune2(shorter,longer);
filtered = [f1 prune2(f1, longer)];

function filtered = prune2(shorter, longer)
filtered = {};
for i = 1:size(longer, 2)
    tested = longer{i};
    ok = 0;
    for j = 1:size(shorter, 2)
        sh = shorter{j}.sequence;
        if size(tested.sequence,2) == size(sh,2)+1 && (cellEq(tested.sequence(2:end),sh) | cellEq(tested.sequence(2:end),sh(end:-1:1)))
            ok = 1;
            break;
        end
    end
    if ok == 1
        if tested.symmetryFlag == 1
            filtered{end+1} = tested;
        else
            for j = 1:size(shorter, 2)
                sh = shorter{j}.sequence;
                if size(tested.sequence,2) == size(sh,2)+1 && (cellEq(tested.sequence(1:end-1),sh) | cellEq(tested.sequence(1:end-1),sh(end:-1:1)))
                    filtered{end+1} = tested;
                    break;
                end
            end
        end
    end
end

function out = cellEq(a,b)
out = 1;
for i = 1:size(a,2)
    if a{i} ~= b{i}
       out = 0;
       break;
    end
end